import 'dart:async';
import 'dart:io';

class WSController {
  String wsUrl;
  bool wsClosed = false;



  static WSController? _singleton;

  StreamController<String> streamController = StreamController.broadcast(sync: true);


  WebSocket? channel;

  factory WSController(wsUrl) {
    return _singleton ??= WSController._internal(wsUrl: wsUrl);
  }

  WSController._internal({required this.wsUrl}) {
    initWebSocketConnection();
  }

  initWebSocketConnection() async {
    if(wsClosed) return;

    print("conecting...");
    channel = await connectWs();
    print("socket connection initialized");
    channel?.done.then((dynamic _) => _onDisconnected());
    broadcastNotifications();
  }

  broadcastNotifications() {
    channel?.listen((streamData) {
      streamController.add(streamData);
    }, onDone: () {
      print("conecting aborted");
      initWebSocketConnection();
    }, onError: (e) {
      print('Server error: $e');
      initWebSocketConnection();
    });
  }

  connectWs() async{
    try {
      return await WebSocket.connect(wsUrl);
    } catch  (e) {
      print("Error! can not connect WS connectWs " + e.toString());
      await Future.delayed(const Duration(milliseconds: 10000));
      return await connectWs();
    }

  }

  closeWs() async {
    wsClosed = true;
    await channel?.close();
  }

  void _onDisconnected() {
    initWebSocketConnection();
  }
}